def to_zero_one_decimal(value):
    if value > 1:
        return value / 100


def print_results(results):
    print("\nThe analysis was completed with Success!\n\nHere are the Results:\n")
    result_values = results["analysis"]

    print(f"SQC: {result_values['sqc']['sqc']}")

    print("\nCharacteristics before weighting:\n")
    for key_c, value_c in result_values["weighted_characteristics"]["sqc"].items():
        print("\t", key_c, "=", f"{to_zero_one_decimal(value_c)}")

    print("\nCharacteristics:\n")
    for key_c, value_c in result_values["characteristics"].items():
        print("\t", key_c, "=", f"{value_c}\n")
        print("\t Subcharacteristics after weighting:")
        for key_sc, value_sc in result_values["weighted_subcharacteristics"][
            key_c
        ].items():
            print("\t\t", key_sc, "=", f"{to_zero_one_decimal(value_sc)}")
        print("\n")

    print("\nSubcharacteristics:\n")
    for key_sc, value_sc in result_values["subcharacteristics"].items():
        print("\t", key_sc, "=", f"{value_sc}\n")
        print("\t Measures after weighting:")
        for key_m, value_m in result_values["weighted_measures"][key_sc].items():
            print("\t\t", key_m, "=", f"{to_zero_one_decimal(value_m)}")
        print("\n")


def validade_analysis_response(status_code, response_json):
    if status_code == 201 or status_code == 200:
        print_results(response_json)
    else:
        if response_json is not None and "error" in response_json.keys():
            print("Error: ", response_json["error"])
        else:
            print("Error while making analysis")



def get_entity(response, entity_name, entity_id, history):
    if entity_id:
        extracted_data, headers, data = \
            get_entity_id(response, entity_name, history)
    else:
        extracted_data, headers, data = \
            get_without_entity_id(response, entity_name, history)

    return extracted_data, headers, data


def get_entity_id(response, entity_name, history):
    extracted_data = []

    if history:
        data = response.json().get("history")
        headers = ['Id', 'History Id', 'Value', 'Created at']
        for entity_data in data:
            extracted_data.append([
                entity_data['metric_id' if entity_name == 'metrics' else 'measure_id'],
                entity_data['id'],
                entity_data['value'],
                entity_data['created_at'],
            ])
    else:
        data = response.json()
        headers = ['Name', 'Value', 'Created at']
        extracted_data = [[
            data['name'],
            data['latest']['value'],
            data['latest']['created_at'],
        ]]

    return extracted_data, headers, data


def get_without_entity_id(response, entity_name, history):
    extracted_data = []

    if history:
        headers = ['Id', 'History Id', 'Name', 'Created at']
        data = response.json().get("results")
        for entity_data in data:
            for history_data in entity_data["history"]:
                extracted_data.append([
                    history_data['metric_id' if entity_name == 'metrics' else 'measure_id'],
                    history_data['id'],
                    history_data['value'],
                    history_data['created_at'],
                ])

    else:
        headers = ['Name', 'Value', 'Created at']
        data = response.json().get("results")
        for entity_data in data:
            extracted_data.append([
                entity_data['name'],
                entity_data['latest']['value'],
                entity_data['latest']['created_at'],
            ])

    return extracted_data, headers, data



# import subprocess


# def capture(command):
#     proc = subprocess.Popen(
#         command,
#         stdout=subprocess.PIPE,
#         stderr=subprocess.PIPE,
#     )
#     out, err = proc.communicate()
#     return out, err, proc.returncode


# def test_available_return_code_invalid():
#     out, _, returncode = capture(["measuresoftgram", "available"])

#     assert returncode == 1



from src.cli.exceptions import exceptions
import json
import math
import os


REQUIRED_SONAR_JSON_KEYS = ["paging", "baseComponent", "components"]

REQUIRED_SONAR_BASE_COMPONENT_KEYS = [
    "id",
    "key",
    "name",
    "qualifier",
    "measures",
]


def file_reader(absolute_path):
    check_file_extension(absolute_path)

    json_data = open_json_file(absolute_path)

    check_sonar_format(json_data)

    check_metrics_values(json_data)

    return json_data["components"]


def folder_reader(absolute_path):
    try:
        os.chdir(absolute_path)
        files_in_dir = os.listdir(absolute_path)
        components = []
        files = []

        for file_path in files_in_dir:
            try:
                components.append(file_reader(file_path))
                files.append(f'{absolute_path.split("/")[-1]}/{file_path}')
            except exceptions.MeasureSoftGramCLIException as error:
                print("Warning: ", error, f"passing {file_path}...")

        os.chdir('..')
    except FileNotFoundError:
        raise FileNotFoundError

    return components, files


def open_json_file(absolute_path):
    try:
        with open(absolute_path, "r") as file:
            return json.load(file)
    except FileNotFoundError:
        raise exceptions.FileNotFound("The file was not found")
    except OSError as error:
        raise exceptions.UnableToOpenFile(f"Failed to open the file. {error}")
    except json.JSONDecodeError as error:
        raise exceptions.InvalidMetricsJsonFile(
            f"Failed to decode the JSON file. {error}"
        )


def get_missing_keys_str(attrs, required_attrs):
    missing_keys = []

    for req_key in required_attrs:
        if req_key not in attrs:
            missing_keys.append(req_key)

    return ", ".join(missing_keys)


def check_sonar_format(json_data):
    attributes = list(json_data.keys())
    missing_keys = get_missing_keys_str(attributes, REQUIRED_SONAR_JSON_KEYS)

    if len(missing_keys) > 0:
        raise exceptions.InvalidMetricsJsonFile(
            f"Invalid Sonar JSON keys. Missing keys are: {missing_keys}"
        )

    base_component = json_data["baseComponent"]
    base_component_attrs = list(base_component.keys())
    missing_keys = get_missing_keys_str(
        base_component_attrs, REQUIRED_SONAR_BASE_COMPONENT_KEYS
    )

    if len(missing_keys) > 0:
        raise exceptions.InvalidMetricsJsonFile(
            f"Invalid Sonar baseComponent keys. Missing keys are: {missing_keys}"
        )

    if len(json_data["components"]) == 0:
        raise exceptions.InvalidMetricsJsonFile(
            "Invalid Sonar JSON components value. It must have at least one component"
        )


def check_file_extension(file_name):
    if file_name.split(".")[-1] != "json":
        raise exceptions.InvalidMetricsJsonFile("Only JSON files are accepted.")


def raise_invalid_metric(key, metric):
    raise exceptions.InvalidMetricException(
        'Invalid metric value in "{}" component for the "{}" metric'.format(key, metric)
    )


def check_metrics_values(json_data):
    try:
        for component in json_data["components"]:
            for measure in component["measures"]:
                value = measure["value"]

                try:
                    if value is None or math.isnan(float(value)):
                        raise_invalid_metric(component["key"], measure["metric"])
                except (ValueError, TypeError):
                    raise_invalid_metric(component["key"], measure["metric"])
    except KeyError:
        raise exceptions.InvalidMetricsJsonFile(
            "Failed to validate Sonar JSON metrics. Please check if the file is a valid Sonar JSON"
        )


def validate_metrics_post(response_status):
    if 200 <= response_status <= 299:
        return 'OK: Data sent successfully'

    return \
        f'FAIL: The host service server returned a {response_status} error. Trying again'



from io import StringIO
from src.cli.commands import parse_show


class DummyResponse:
    def __init__(self, status_code, mocked_data):
        self.status_code = status_code
        self.res = mocked_data

    def json(self):
        return self.res


def test_pre_configs_show(mocker):
    mocked_pre_config = {
        "_id": "62656d15f354349ee4abfc7b",
        "name": "TESTa essa joça",
        "characteristics": {
            "reliability": {
                "expected_value": 70,
                "weight": 50,
                "subcharacteristics": ["testing_status"],
                "weights": {"testing_status": 100.0},
            },
            "maintainability": {
                "expected_value": 30,
                "weight": 50,
                "subcharacteristics": ["modifiability"],
                "weights": {"modifiability": 100.0},
            },
        },
        "subcharacteristics": {
            "testing_status": {
                "weights": {"passed_tests": 100.0},
                "measures": ["passed_tests"],
            },
            "modifiability": {
                "weights": {"non_complex_file_density": 100.0},
                "measures": ["non_complex_file_density"],
            },
        },
        "measures": ["passed_tests", "non_complex_file_density"],
        "created_at": "2022-04-24 15:30:29+00:00",
    }

    mocker.patch("requests.get", return_value=DummyResponse(200, mocked_pre_config))

    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        parse_show("abcd")

        expected_lines = [
            "Name: TESTa essa joça",
            "ID: 62656d15f354349ee4abfc7b",
            "Created at: 04/24/2022 12:30:29",
            "reliability (weigth: 50)",
            "testing_status (weigth: 100.0)",
            "passed_tests (weigth: 100.0)",
            "maintainability (weigth: 50)",
            "modifiability (weigth: 100.0)",
            "non_complex_file_density (weigth: 100.0)",
        ]

        for line in expected_lines:
            assert line in fake_out.getvalue()


def test_error_in_pre_config_list(mocker):
    error_res = {"error": "Generic Not Found Error"}

    mocker.patch("requests.get", return_value=DummyResponse(404, error_res))

    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        parse_show("abcd")

        assert error_res["error"] in fake_out.getvalue()



from src.cli.commands import print_results, validade_analysis_response
from io import StringIO


RESULTS = {
    "analysis": {
        "sqc": {"sqc": 0.6165241607725739},
        "subcharacteristics": {
            "modifiability": 0.5,
            "testing_status": 0.7142857142857143,
        },
        "characteristics": {
            "maintainability": 0.5,
            "reliability": 0.7142857142857143,
        },
        "weighted_characteristics": {
            "sqc": {
                "maintainability": 0.5,
                "reliability": 0.7142857142857143,
            }
        },
        "weighted_subcharacteristics": {
            "maintainability": {
                "modifiability": 0.5,
            },
            "reliability": {
                "testing_status": 0.7142857142857143,
            },
        },
        "weighted_measures": {
            "modifiability": {
                "m1": 0.5,
            },
            "testing_status": {
                "m1": 0.7142857142857143,
            },
        },
    }
}

ERROR_MESSAGE = {"error": "Pre-Config is not a valid ID"}


def test_print_results(mocker):

    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        print_results(RESULTS)

        out = fake_out.getvalue()

        assert "The analysis was completed with Success!" in out
        assert "SQC: 0.6165241607725739" in out
        assert "maintainability = 0.5"
        assert "reliability = 0.7142857142857143" in out
        assert "modifiability = 0.5" in out
        assert "testing_status = 0.7142857142857143" in out


def test_validate_analysis_response_success(mocker):

    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        validade_analysis_response(201, RESULTS)

        out = fake_out.getvalue()

        assert "The analysis was completed with Success!" in out
        assert "SQC: 0.6165241607725739" in out
        assert "maintainability = 0.5"
        assert "reliability = 0.7142857142857143" in out
        assert "modifiability = 0.5" in out
        assert "testing_status = 0.7142857142857143" in out


def test_validate_analysis_response_error(mocker):
    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        validade_analysis_response(404, ERROR_MESSAGE)

        assert "Pre-Config is not a valid ID" in fake_out.getvalue()



import requests

BASE_URL = "http://localhost:5000/"


def parse_available():
    available_pre_configs = requests.get(
        BASE_URL + "available-pre-configs", headers={"Accept": "application/json"}
    ).json()

    print(
        "\nThese are all items available in the MeasureSoftGram database in the following order:\
            \nCharacteristics -> Subcharacteristics -> Measures -> Necessary Metrics\
            \n\n You can use these items to create a pre configuration"
    )

    characteristics = available_pre_configs["characteristics"]
    subcharacteristics = available_pre_configs["subcharacteristics"]
    measures = available_pre_configs["measures"]

    for characteristic in characteristics:
        print(f"\n\t{characteristics[characteristic]['name']}:")
        for subcharacteristic in subcharacteristics:
            if (
                characteristic
                in subcharacteristics[subcharacteristic]["characteristics"]
            ):
                print(f"\t\t{subcharacteristics[subcharacteristic]['name']}:")
                for measure in measures:
                    if subcharacteristic in measures[measure]["subcharacteristics"]:
                        print(f"\t\t\t{measures[measure]['name']}:")
                        print(f"\t\t\t\t{', '.join(measures[measure]['metrics'])}")



import requests
from src.cli.utils import pretty_date_str

BASE_URL = "http://localhost:5000/"


def parse_list():
    response = requests.get(
        BASE_URL + "/pre-configs",
        headers={"Accept": "application/json"},
    )

    pre_configs = response.json()

    if not 200 <= response.status_code <= 299:
        print("Error: an error occurred while fetching your pre configurations")
        return

    print(
        "{:<30} {:<35} {:<30} {:<10}".format("ID", "Name", "Created at", "Metrics file")
    )

    for pre_config in pre_configs:
        created_at = pretty_date_str(pre_config["created_at"])
        pre_config_name = pre_config["name"] if pre_config["name"] else "-"

        print(
            "{:<30} {:<35} {:<30} {:<10}".format(
                pre_config["_id"], pre_config_name, created_at, "-"
            )
        )



from src.cli.exceptions.exceptions import (
    FileNotFound,
    InvalidMeasuresoftgramFormat,
    InvalidMetricException,
    InvalidMetricsJsonFile,
    InvalidWeight,
    MeasureSoftGramCLIException,
    UnableToOpenFile,
    UnableToReadFile
)



import pytz
from termcolor import colored
from datetime import datetime


def pretty_date_str(date_str, format="%m/%d/%Y %H:%M:%S", timezone="Brazil/East"):
    date_time = datetime.fromisoformat(date_str)

    date_time = date_time.astimezone(pytz.timezone(timezone))

    return date_time.strftime(format)


def check_host_url(host_url):
    return host_url if host_url.endswith("/") else host_url + "/"


def print_import_files(files):
    """
        Importing files:
            - sonarmetrics/file1.json
            - sonarmetrics/file2.json
            - sonarmetrics/file3.json

        Sending the file data:
    """
    print('\n\tImporting files:')

    for file in files:
        print(f'\t\t- {file}')

    print('\n\tSending the file data:')


def print_status_import_file(file, message, trying_idx):
    """
            - sonarmetrics/file1.json
            OK: Data sent successfully
    """
    print(f'\t\t- [{trying_idx}] {file}')

    if 'OK:' in message:
        print(colored(f'\t\t{message}\n', 'green'))
    else:
        print(colored(f'\t\t{message}\n', 'red'))



import pytest
from io import StringIO
from src.cli.commands import parse_change_name


class DummyResponse:
    def __init__(self, status_code, res_data):
        self.status_code = status_code
        self.res_data = res_data

    def json(self):
        return self.res_data


def test_change_name_sucess(mocker):
    res_data = {
        "_id": "6261b76c974ddbc76bdea7af",
        "name": "pre-config-one",
        "characteristics": {},
        "subcharacteristics": {},
        "measures": [],
        "created_at": "2022-04-21 19:58:36+00:00",
    }

    mocker.patch("requests.patch", return_value=DummyResponse(200, res_data))

    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        parse_change_name("6261b76c974ddbc76bdea7af", "pre-config-1")

        expected = (
            'Your Pre Configuration name was succesfully changed to "pre-config-one"'
        )

        assert expected in fake_out.getvalue()


@pytest.mark.parametrize(
    "status_code, error_msg",
    [
        (
            404,
            "There was an ERROR while changing your Pre Configuration name: "
            + "There is no pre configurations with ID 6261b76c974ddbc76bdea7af",
        ),
        (
            422,
            "There was an ERROR while changing your Pre Configuration name: "
            + "The pre config name is already in use",
        ),
    ],
)
def test_change_name_error(mocker, status_code, error_msg):
    res_data = {"error": error_msg}

    mocker.patch("requests.patch", return_value=DummyResponse(status_code, res_data))

    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        parse_change_name("6261b76c974ddbc76bdea7af", "pre-config-1")

        assert error_msg in fake_out.getvalue()



from src.cli.commands.parse_import.parse_import import parse_import
from src.cli.commands.parse_analysis.parse_analysis import parse_analysis
from src.cli.commands.parse_create.parse_create import parse_create
from src.cli.commands.parse_change_name.parse_change_name import (
    parse_change_name
)
from src.cli.commands.parse_get_entity.parse_get_entity import parse_get_entity
from src.cli.commands.parse_available.parse_available import parse_available
from src.cli.commands.parse_show.parse_show import parse_show
from src.cli.commands.parse_list.parse_list import parse_list

from src.cli.commands.parse_create.utils import (
    check_file_extension,
    check_in_keys,
    open_json_file,
    pre_config_file_reader,
    read_file_characteristics,
    read_file_measures,
    read_file_sub_characteristics,
    round_sum_of_weights,
    validate_core_available,
    validate_file_characteristics,
    validate_file_measures,
    validate_file_sub_characteristics,
    validate_pre_config_post,
    validate_sum_of_weights,
    validate_weight_parameter,
    validate_weight_value
)
from src.cli.commands.parse_get_entity.utils import (
    get_entity,
    get_entity_id,
    get_without_entity_id
)
from src.cli.commands.parse_analysis.results import (
    print_results,
    to_zero_one_decimal,
    validade_analysis_response
)



import json
import requests

from src.cli.exceptions.exceptions import MeasureSoftGramCLIException
from src.cli.commands.parse_create.utils import (
    validate_pre_config_post,
    pre_config_file_reader
)
from src.config.settings import BASE_URL


def parse_create(file_path):
    available_pre_config = requests.get(
        BASE_URL + "available-pre-configs",
        headers={"Accept": "application/json"}
    ).json()

    try:
        pre_config = pre_config_file_reader(
            r"{}".format(file_path),
            available_pre_config,
        )
    except MeasureSoftGramCLIException as error:
        print("Error: ", error)
        return

    response = requests.post(BASE_URL + "pre-configs", json=pre_config)

    saved_pre_config = json.loads(response.text)

    validate_pre_config_post(response.status_code, saved_pre_config)



import json

from tabulate import tabulate
from src.cli.commands.parse_get_entity.utils import get_entity

from src.cli.utils import check_host_url
from src.clients.service_client import ServiceClient
from src.config.settings import SUPPORTED_FORMATS


def parse_get_entity(
    entity_name,
    entity_id,
    host_url,
    organization_id,
    repository_id,
    output_format,
    history,
):
    if output_format not in SUPPORTED_FORMATS:
        print((
            "Output format not supported. "
            f"Supported formats: {SUPPORTED_FORMATS}"
        ))
        return

    host_url = check_host_url(host_url)
    host_url += (
        'api/v1/'
        f'organizations/{organization_id}/'
        f'repository/{repository_id}/'
        f'{"history/" if history else ""}'
        f'{entity_name}/'
        f'{entity_id if entity_id else ""}'
    )
    response = ServiceClient.get_entity(host_url)

    extracted_data, headers, data = get_entity(
        response,
        entity_name,
        entity_id,
        history
    )

    if output_format == 'tabular':
        print(tabulate(extracted_data, headers=headers))
    elif output_format == 'json':
        print(json.dumps(data))



import subprocess


def capture(command):
    proc = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    out, err = proc.communicate()

    return out, err, proc.returncode


# def test_no_env():
#     out, err, returncode = capture(["measuresoftgram", "create", "path"])
#     assert returncode == 1
    # FIXME
    # assert "Name or service not known" in err.decode("utf-8")



from src.cli.exceptions import exceptions
from src.cli.jsonReader import check_file_extension, open_json_file


def pre_config_file_reader(absolute_path, available_pre_configs):

    try:
        core_format = available_pre_configs
        check_file_extension(absolute_path)

        pre_config_json_file = open_json_file(absolute_path)

        pre_config_file_name = pre_config_json_file.get("pre_config_name", None)

        validate_file_characteristics(pre_config_json_file)
        validate_file_sub_characteristics(pre_config_json_file)
        validate_file_measures(pre_config_json_file)

        file_characteristics = read_file_characteristics(pre_config_json_file)
        file_sub_characteristics = read_file_sub_characteristics(pre_config_json_file)
        file_measures = read_file_measures(pre_config_json_file)

        validate_core_available(
            core_format, file_characteristics, file_sub_characteristics
        )

        pre_config = {
            "name": pre_config_file_name,
            "characteristics": file_characteristics,
            "subcharacteristics": file_sub_characteristics,
            "measures": file_measures,
        }

        return pre_config
    except KeyError as error:
        raise exceptions.InvalidMeasuresoftgramFormat(
            f"Invalid JSON format, key {error} does not exist"
        )


def read_file_characteristics(pre_config_json_file):

    characteristics = {}
    weights_auxiliar = {}
    char_sub_list = []

    for characteristic in pre_config_json_file["characteristics"]:

        characteristic_auxiliar = {"weight": characteristic["weight"]}
        weights_auxiliar = {}

        for subcharacteristic in characteristic["subcharacteristics"]:
            char_sub_list.append(subcharacteristic["name"])

            weights_auxiliar.update(
                {subcharacteristic["name"]: subcharacteristic["weight"]}
            )

            characteristic_auxiliar.update(
                {"subcharacteristics": char_sub_list, "weights": weights_auxiliar}
            )
        char_sub_list = []

        characteristics.update({characteristic["name"]: characteristic_auxiliar})

    return characteristics


def read_file_sub_characteristics(pre_config_json_file):

    subcharacteristics = {}
    weights_auxiliar = {}
    sub_mea_list = []

    for characteristic in pre_config_json_file["characteristics"]:
        for subcharacteristic in characteristic["subcharacteristics"]:

            weights_auxiliar = {}
            subcharacteristic_auxiliar = {
                "weights": {characteristic["name"]: subcharacteristic["weight"]},
            }

            for measure in subcharacteristic["measures"]:
                sub_mea_list.append(measure["name"])

                weights_auxiliar.update({measure["name"]: measure["weight"]})
                subcharacteristic_auxiliar.update(
                    {"weights": weights_auxiliar, "measures": sub_mea_list}
                )
            sub_mea_list = []

            subcharacteristics.update(
                {subcharacteristic["name"]: subcharacteristic_auxiliar}
            )

    return subcharacteristics


def read_file_measures(pre_config_json_file):

    measures = []

    for characteristic in pre_config_json_file["characteristics"]:
        for subcharacteristic in characteristic["subcharacteristics"]:
            for measure in subcharacteristic["measures"]:

                measures.append(measure["name"])

    return measures


def validate_file_characteristics(pre_config_json_file):

    sum_of_characteristics_weights = 0
    characteristics_names = []

    for characteristic in pre_config_json_file["characteristics"]:

        characteristics = characteristic.keys()

        check_in_keys(
            "name",
            characteristics,
            exceptions.UnableToReadFile,
            "Expected characteristic name field.",
        )

        check_in_keys(
            "weight",
            characteristics,
            exceptions.InvalidWeight,
            "{} characteristic does not have weight field defined.".format(
                characteristic["name"]
            ),
        )

        validate_weight_parameter(
            characteristic["weight"],
            exceptions.InvalidWeight,
            "{} does not have weight value inside parameters (0 to 100).".format(
                characteristic["name"]
            ),
        )

        if "weight" in characteristics:
            sum_of_characteristics_weights = (
                sum_of_characteristics_weights + characteristic["weight"]
            )

        check_in_keys(
            "subcharacteristics",
            characteristics,
            exceptions.UnableToReadFile,
            "{} does not have subcharacteristics field defined.".format(
                characteristic["name"]
            ),
        )

        if (
            characteristic["subcharacteristics"] is None
            or len(characteristic["subcharacteristics"]) == 0
        ):
            raise exceptions.UnableToReadFile(
                "{} needs to have at least one subcharacteristic defined.".format(
                    characteristic["name"]
                )
            )

        characteristics_names.append(characteristic["name"])

    if validate_sum_of_weights(sum_of_characteristics_weights) is False:
        raise exceptions.UnableToReadFile(
            "The sum of characteristics weights of is not 100"
        )

    return True


def validate_file_sub_characteristics(pre_config_json_file):

    sub_characteristics_names = []

    for characteristic in pre_config_json_file["characteristics"]:

        sum_of_subcharacteristics_weights = 0

        for subcharacteristic in characteristic["subcharacteristics"]:

            subcharacteristics = subcharacteristic.keys()

            check_in_keys(
                "name",
                subcharacteristics,
                exceptions.UnableToReadFile,
                "Expected sub-characteristic name field.",
            )

            check_in_keys(
                "weight",
                subcharacteristics,
                exceptions.InvalidWeight,
                '"{}" subcharacteristic does not have weight field defined.'.format(
                    subcharacteristic["name"]
                ),
            )

            validate_weight_parameter(
                subcharacteristic["weight"],
                exceptions.InvalidWeight,
                '"{}" subcharacteristics does not have weight value inside parameters (0 to 100).'.format(
                    subcharacteristic["name"]
                ),
            )

            if "weight" in subcharacteristics:
                sum_of_subcharacteristics_weights = (
                    sum_of_subcharacteristics_weights + subcharacteristic["weight"]
                )

            check_in_keys(
                "measures",
                subcharacteristics,
                exceptions.UnableToReadFile,
                '"{}" subcharacteristic does not have measures field defined.'.format(
                    subcharacteristic["name"]
                ),
            )

            if (
                subcharacteristic["measures"] is None
                or len(subcharacteristic["measures"]) == 0
            ):
                raise exceptions.UnableToReadFile(
                    '"{}" subcharacteristic needs to have at least one measure defined.'.format(
                        subcharacteristic["name"]
                    )
                )

            sub_characteristics_names.append(subcharacteristic["name"])

        sum_of_subcharacteristics_weights = round_sum_of_weights(
            sum_of_subcharacteristics_weights
        )

        if validate_sum_of_weights(sum_of_subcharacteristics_weights) is False:
            raise exceptions.InvalidWeight(
                "The sum of subcharacteristics weights is not 100"
            )

    return True


def validate_file_measures(pre_config_json_file):

    measures_names = []

    for characteristic in pre_config_json_file["characteristics"]:
        for subcharacteristic in characteristic["subcharacteristics"]:

            sum_of_measures_weights = 0

            for measure in subcharacteristic["measures"]:

                measures = measure.keys()

                check_in_keys(
                    "name",
                    measures,
                    exceptions.UnableToReadFile,
                    "Expected measure name field.",
                )

                check_in_keys(
                    "weight",
                    measures,
                    exceptions.InvalidWeight,
                    "{} measure does not have weight field defined.".format(
                        measure["name"]
                    ),
                )

                validate_weight_parameter(
                    measure["weight"],
                    exceptions.InvalidWeight,
                    "{} measure does not have weight value inside parameters (0 to 100).".format(
                        measure["name"]
                    ),
                )

                if "weight" in measures:
                    sum_of_measures_weights = (
                        sum_of_measures_weights + measure["weight"]
                    )

                measures_names.append(measure["name"])

            if validate_sum_of_weights(sum_of_measures_weights) is False:
                raise exceptions.InvalidWeight("The sum of measures weights is not 100")

    return True


def validate_weight_parameter(weight, exception, exception_description):

    if not validate_weight_value(weight):
        raise exception(exception_description)

    return True


def check_in_keys(key_for_check, keys, exception, exception_description):

    if key_for_check not in keys:
        raise exception(exception_description)

    return True


def round_sum_of_weights(sum_weights):

    if 0 < round(100 - sum_weights, 2) <= 0.01:
        sum_weights = 100

    return sum_weights


def validate_sum_of_weights(sum_weights):

    sum_weights = round_sum_of_weights(sum_weights)

    if sum_weights != 100.0:
        return False

    return True


def validate_core_available(
    available_pre_configs, file_characteristics, file_subcharacteristics
):
    core_characteristics = list(available_pre_configs["characteristics"].keys())
    characteristics = list(file_characteristics.keys())

    core_characteristics.sort()
    characteristics.sort()

    for item in [x for x in characteristics if x not in core_characteristics]:
        raise exceptions.UnableToReadFile(
            'The characteristic "{}" is not in MeasureSoftGram database'.format(item)
        )

    for char in file_characteristics.keys():

        for item in [
            x
            for x in file_characteristics[char]["subcharacteristics"]
            if x
            not in available_pre_configs["characteristics"][char]["subcharacteristics"]
        ]:
            raise exceptions.UnableToReadFile(
                'The subcharacteristic "{}" is in a wrong characteristic'.format(item)
                + "or it is not in MeasureSoftgram database"
            )

    for sub in file_subcharacteristics.keys():

        for item in [
            x
            for x in file_subcharacteristics[sub]["measures"]
            if x not in available_pre_configs["subcharacteristics"][sub]["measures"]
        ]:
            raise exceptions.UnableToReadFile(
                'The measure "{}" is in a wrong subcharacteristic or it is not in MeasureSoftgram database'.format(
                    item
                )
            )

    return True


def validate_weight_value(weight):
    return 0 < weight <= 100


def validate_pre_config_post(status_code, response):
    if status_code == 201:
        print(
            f"\nYour Pre Configuration was created with success!\nPre Configuration ID: {response['_id']}"
        )
    else:
        print(
            f"\nThere was an ERROR while creating your Pre Configuration:  {response['error']}"
        )



import signal
import subprocess
from time import sleep
# import pytest


def capture(command):
    proc = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    sleep(0.5)

    proc.send_signal(signal.SIGINT)

    out, err = proc.communicate()

    return out, err, proc.returncode


# @pytest.mark.xfail
# def test_sigint():
#     out, err, returncode = capture(["measuresoftgram", "create"])

#     print(out)
#     print(err)
#     assert returncode == 0
#     assert "\n\nExiting MeasureSoftGram..." in out.decode("utf-8")



from src.cli.jsonReader.jsonReader import (
    check_file_extension,
    check_metrics_values,
    check_sonar_format,
    file_reader,
    folder_reader,
    get_missing_keys_str,
    open_json_file,
    raise_invalid_metric,
    validate_metrics_post,
    REQUIRED_SONAR_JSON_KEYS,
    REQUIRED_SONAR_BASE_COMPONENT_KEYS
)



import subprocess


def capture(command):
    proc = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    out, err = proc.communicate()
    return out, err, proc.returncode


def test_show_help():
    out, err, returncode = capture(["measuresoftgram", "-h"])
    assert returncode == 0
    assert "usage: measuresoftgram [-h]" in out.decode("utf-8")

    out, err, returncode = capture(["measuresoftgram"])
    assert returncode == 0
    assert "usage: measuresoftgram [-h]" in out.decode("utf-8")



import pytest
import src.cli.commands as create
from src.cli.exceptions import exceptions
from tests.test_helpers import read_json


def test_pre_config_file_reader():
    available_pre_config = read_json("tests/unit/data/measuresoftgramCoreFormat.json")

    create.pre_config_file_reader(
        "tests/unit/data/measuresoftgramPreConfig.json", available_pre_config
    )

    assert {
        "characteristics": {
            "reliability": {
                "weight": 50.0,
                "subcharacteristics": ["testing_status"],
                "weights": {"testing_status": 100.0},
            },
            "maintainability": {
                "weight": 50.0,
                "subcharacteristics": ["modifiability"],
                "weights": {"modifiability": 100.0},
            },
        },
        "subcharacteristics": {
            "testing_status": {
                "weights": {
                    "passed_tests": 33.33,
                    "test_builds": 33.33,
                    "test_coverage": 33.33,
                },
                "measures": ["passed_tests", "test_builds", "test_coverage"],
            },
            "modifiability": {
                "weights": {
                    "non_complex_file_density": 50.0,
                    "commented_file_density": 30.0,
                    "duplication_absense": 20.0,
                },
                "measures": [
                    "non_complex_file_density",
                    "commented_file_density",
                    "duplication_absense",
                ],
            },
        },
        "measures": [
            "passed_tests",
            "test_builds",
            "test_coverage",
            "non_complex_file_density",
            "commented_file_density",
            "duplication_absense",
        ],
    }


def test_valid_read_file_characteristics():
    file_pre_config = read_json("tests/unit/data/measuresoftgramPreConfig.json")
    characteristics = create.read_file_characteristics(file_pre_config)

    assert characteristics == {
        "reliability": {
            "weight": 50.0,
            "subcharacteristics": ["testing_status"],
            "weights": {"testing_status": 100.0},
        },
        "maintainability": {
            "weight": 50.0,
            "subcharacteristics": ["modifiability"],
            "weights": {"modifiability": 100.0},
        },
    }


def test_valid_validate_file_characteristics():
    file_pre_config = read_json("tests/unit/data/measuresoftgramPreConfig.json")
    characteristics = create.validate_file_characteristics(file_pre_config)

    assert characteristics


def test_invalid_validate_file_characteristics():

    file_characteristics_without_weights = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_characteristics_without_weights["characteristics"][0].pop("weight")

    with pytest.raises(exceptions.InvalidWeight):
        create.validate_file_characteristics(file_characteristics_without_weights)

    file_characteristics_without_name = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_characteristics_without_name["characteristics"][0].pop("name")

    with pytest.raises(exceptions.UnableToReadFile):
        create.validate_file_characteristics(file_characteristics_without_name)

    file_characteristics_without_subc = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_characteristics_without_subc["characteristics"][0].pop("subcharacteristics")

    with pytest.raises(exceptions.UnableToReadFile):
        create.validate_file_characteristics(file_characteristics_without_subc)


def test_valid_read_file_sub_characteristics():
    file_subcharacteristics = read_json("tests/unit/data/measuresoftgramPreConfig.json")
    subcharacteristics = create.read_file_sub_characteristics(file_subcharacteristics)

    assert subcharacteristics == {
        "testing_status": {
            "weights": {
                "passed_tests": 33.33,
                "test_builds": 33.33,
                "test_coverage": 33.33,
            },
            "measures": ["passed_tests", "test_builds", "test_coverage"],
        },
        "modifiability": {
            "weights": {
                "non_complex_file_density": 50.0,
                "commented_file_density": 30.0,
                "duplication_absense": 20.0,
            },
            "measures": [
                "non_complex_file_density",
                "commented_file_density",
                "duplication_absense",
            ],
        },
    }


def test_valid_validate_file_sub_characteristics():
    file_subcharacteristics = read_json("tests/unit/data/measuresoftgramPreConfig.json")
    subcharacteristics = create.validate_file_sub_characteristics(
        file_subcharacteristics
    )

    assert subcharacteristics


def test_invalid_validate_file_sub_characteristics():

    file_without_name_subcharacteristics = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_without_name_subcharacteristics["characteristics"][0]["subcharacteristics"][
        0
    ].pop("name")

    with pytest.raises(exceptions.UnableToReadFile):
        create.validate_file_sub_characteristics(file_without_name_subcharacteristics)

    file_without_weight_subcharacteristics = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_without_weight_subcharacteristics["characteristics"][0]["subcharacteristics"][
        0
    ].pop("weight")

    with pytest.raises(exceptions.InvalidWeight):
        create.validate_file_sub_characteristics(file_without_weight_subcharacteristics)

    file_without_measures_subcharacteristics = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_without_measures_subcharacteristics["characteristics"][0][
        "subcharacteristics"
    ][0].pop("measures")

    with pytest.raises(exceptions.UnableToReadFile):
        create.validate_file_sub_characteristics(
            file_without_measures_subcharacteristics
        )

    file_empty_measures_subcharacteristics = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_empty_measures_subcharacteristics["characteristics"][0]["subcharacteristics"][
        0
    ]["measures"] = []

    with pytest.raises(exceptions.UnableToReadFile):
        create.validate_file_sub_characteristics(file_empty_measures_subcharacteristics)


def test_valid_read_file_measures():
    file_pre_config = read_json("tests/unit/data/measuresoftgramPreConfig.json")

    measures = create.read_file_measures(file_pre_config)

    assert measures == [
        "passed_tests",
        "test_builds",
        "test_coverage",
        "non_complex_file_density",
        "commented_file_density",
        "duplication_absense",
    ]


def test_valid_validate_file_measures():
    file_pre_config = read_json("tests/unit/data/measuresoftgramPreConfig.json")

    measures = create.validate_file_measures(file_pre_config)

    assert measures


def test_invalid_validate_file_measures():

    file_without_name_measures = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_without_name_measures["characteristics"][0]["subcharacteristics"][0][
        "measures"
    ][0].pop("name")

    with pytest.raises(exceptions.UnableToReadFile):
        create.validate_file_measures(file_without_name_measures)

    file_without_weight_measures = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_without_weight_measures["characteristics"][0]["subcharacteristics"][0][
        "measures"
    ][0].pop("weight")

    with pytest.raises(exceptions.InvalidWeight):
        create.validate_file_measures(file_without_weight_measures)

    file_invalid_weight_measures = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_invalid_weight_measures["characteristics"][0]["subcharacteristics"][0][
        "measures"
    ][0]["weight"] = 120

    with pytest.raises(exceptions.InvalidWeight):
        create.validate_file_measures(file_invalid_weight_measures)

    file_invalid_weight_sum_measures = read_json(
        "tests/unit/data/measuresoftgramPreConfig.json"
    )
    file_invalid_weight_sum_measures["characteristics"][0]["subcharacteristics"][0][
        "measures"
    ][0]["weight"] = 10

    with pytest.raises(exceptions.InvalidWeight):
        create.validate_file_measures(file_invalid_weight_sum_measures)


def test_valid_validate_core_available():
    file_pre_config = read_json("tests/unit/data/measuresoftgramPreConfig.json")

    available_pre_configs = read_json("tests/unit/data/measuresoftgramCoreFormat.json")
    file_characteristics = create.read_file_characteristics(file_pre_config)
    file_subcharacteristics = create.read_file_sub_characteristics(file_pre_config)

    assert create.validate_core_available(
        available_pre_configs, file_characteristics, file_subcharacteristics
    )


def test_invalid_validate_core_available():

    available_pre_configs = read_json("tests/unit/data/measuresoftgramCoreFormat.json")
    file_wrong_characteristics = {
        "usability": {
            "weight": 50,
            "subcharacteristics": ["testing_status"],
            "weights": {"testing_status": 100.0},
        },
        "maintainability": {
            "weight": 50,
            "subcharacteristics": ["modifiability"],
            "weights": {"modifiability": 100.0},
        },
    }
    file_subcharacteristics = (
        {
            "testing_status": {
                "weights": {"passed_tests": 100.0},
                "measures": ["passed_tests"],
            },
            "modifiability": {
                "weights": {"non_complex_file_density": 100.0},
                "measures": ["non_complex_file_density"],
            },
        },
    )

    assert list(available_pre_configs["characteristics"].keys()) != list(
        file_wrong_characteristics.keys()
    )

    with pytest.raises(exceptions.UnableToReadFile):
        create.validate_core_available(
            available_pre_configs, file_wrong_characteristics, file_subcharacteristics
        )

    file_characteristics_wrong_subcharacteristics = {
        "reliability": {
            "weight": 50,
            "subcharacteristics": ["modifiability"],
            "weights": {"testing_status": 100.0},
        },
        "maintainability": {
            "weight": 50,
            "subcharacteristics": ["modifiability"],
            "weights": {"modifiability": 100.0},
        },
    }

    with pytest.raises(exceptions.UnableToReadFile):
        create.validate_core_available(
            available_pre_configs,
            file_characteristics_wrong_subcharacteristics,
            file_subcharacteristics,
        )

    file_characteristics = {
        "reliability": {
            "weight": 50,
            "subcharacteristics": ["testing_status"],
            "weights": {"testing_status": 100.0},
        },
        "maintainability": {
            "weight": 50,
            "subcharacteristics": ["modifiability"],
            "weights": {"modifiability": 100.0},
        },
    }
    file_subcharacteristics_wrong_measures = {
        "testing_status": {
            "weights": {"passed_tests": 100.0},
            "measures": ["non_complex_file_density"],
        },
        "modifiability": {
            "weights": {"non_complex_file_density": 100.0},
            "measures": ["passed_tests"],
        },
    }

    with pytest.raises(exceptions.UnableToReadFile):
        create.validate_core_available(
            available_pre_configs,
            file_characteristics,
            file_subcharacteristics_wrong_measures,
        )


def test_validate_weight_value():
    """
    Test for validate_weight_value function
    """
    assert create.validate_weight_value(0) is False
    assert create.validate_weight_value(50)
    assert create.validate_weight_value(100)
    assert create.validate_weight_value(-30) is False
    assert create.validate_weight_value(120) is False


def test_check_in_keys():
    key_for_check = "name"
    keys = ["weight", "subcharacteristics"]
    exception = exceptions.UnableToReadFile
    exception_description = "teste"

    with pytest.raises(exceptions.UnableToReadFile):
        create.check_in_keys(key_for_check, keys, exception, exception_description)

    keys = ["name", "weight", "subcharacteristics"]

    assert create.check_in_keys(key_for_check, keys, exception, exception_description)


def test_validate_weight_parameter():
    weight = 120
    exception = exceptions.InvalidWeight
    exception_description = "teste"

    with pytest.raises(exceptions.InvalidWeight):
        create.validate_weight_parameter(weight, exception, exception_description)

    weight = 90

    assert create.validate_weight_parameter


def test_round_of_sum_weights():
    sum_weights = 99.99

    sum_weights = create.round_sum_of_weights(sum_weights)
    assert sum_weights == 100


def test_validate_sum_of_weights():
    sum_weights = 100

    validate = create.validate_sum_of_weights(sum_weights)
    assert validate is True

    sum_weights = 99.99

    validate = create.validate_sum_of_weights(sum_weights)
    assert validate is True

    sum_weights = 99

    validate = create.validate_sum_of_weights(sum_weights)
    assert validate is False



class MeasureSoftGramCLIException(Exception):
    """Base MeasureSoftGram CLI Exception"""

    pass


class InvalidMetricException(MeasureSoftGramCLIException):
    """Raised when an invalid metric value is provided"""

    pass


class InvalidMetricsJsonFile(MeasureSoftGramCLIException):
    """Raised when the metrics file is invalid"""

    pass


class FileNotFound(MeasureSoftGramCLIException):
    """Raised when a file is not found"""

    pass


class UnableToOpenFile(MeasureSoftGramCLIException):
    """Raised when a file could not be opened"""

    pass


class UnableToReadFile(MeasureSoftGramCLIException):
    """Raised when a file could not be readed"""

    pass


class InvalidWeight(MeasureSoftGramCLIException):
    """Raised when a invalid weight is provided to the MeasureSoftGram"""

    pass


class InvalidMeasuresoftgramFormat(MeasureSoftGramCLIException):
    """Raised when a invalid format file is provided to the MeasureSoftGram"""

    pass



import requests

from src.config.settings import BASE_URL


def parse_change_name(pre_config_id, new_name):
    response = requests.patch(
        BASE_URL + f"pre-configs/{pre_config_id}", json={"name": new_name}
    )

    response_data = response.json()

    if 200 <= response.status_code <= 299:
        print(
            f'Your Pre Configuration name was succesfully changed to "{response_data["name"]}"'
        )
    else:
        print(
            f"There was an ERROR while changing your Pre Configuration name:  {response_data['error']}"
        )



import json
from urllib.error import HTTPError
import requests

from src.cli.exceptions.exceptions import MeasureSoftGramCLIException
from src.cli.jsonReader import folder_reader, validate_metrics_post
from src.cli.utils import check_host_url, print_import_files, print_status_import_file
from src.clients.service_client import ServiceClient


def parse_import(
    output_origin,
    dir_path,
    language_extension,
    host_url,
    organization_id,
    repository_id
):
    print(f'--> Starting to parser import for {output_origin} output...\n')

    try:
        components, files = folder_reader(r"{}".format(dir_path))
    except (MeasureSoftGramCLIException, FileNotFoundError):
        print("Error: The folder was not found")
        return

    payload = {
        "components": [],
        "language_extension": language_extension,
    }

    host_url = check_host_url(host_url)

    host_url += (
        'api/v1/'
        f'organizations/{organization_id}/'
        f'repository/{repository_id}/'
        'import/sonarqube-metrics/'
    )

    print_import_files(files)

    for idx, component in enumerate(components):
        payload["components"] = component

        for trying_idx in range(3):
            try:
                response = ServiceClient.import_file(host_url, payload)

                message = validate_metrics_post(response.status_code)

                print_status_import_file(files[idx], message, trying_idx + 1)
                break
            except (
                requests.RequestException,
                ConnectionError,
                HTTPError,
                json.decoder.JSONDecodeError
            ):
                print_status_import_file(
                    files[idx],
                    "FAIL: Can't connect to host service.",
                    trying_idx + 1
                )

    print('\nAttempt to save all files in the directory finished!')



import pytest
from src.cli import jsonReader
from src.cli.exceptions import exceptions
# from io import StringIO


@pytest.mark.parametrize("file_name", ["sonar.txt", "sonar.xml", "sonar.jjson"])
def test_invalid_file_extension(file_name):
    """
    Test check_file_extension when an invalid extesion is provided
    """

    with pytest.raises(exceptions.InvalidMetricsJsonFile) as error:
        jsonReader.check_file_extension(file_name)

    assert str(error.value) == "Only JSON files are accepted."


def test_file_reader_success():
    """
    Test file_reader function success case
    """
    EXPECTED_SONAR_JSON_COMPONENTS = [
        {
            "id": "AX9GDsKlZuVL7NjXSAZ4",
            "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:tests/__init__.py",
            "name": "__init__.py",
            "qualifier": "UTS",
            "path": "tests/__init__.py",
            "language": "py",
            "measures": [
                {"metric": "security_rating", "value": "1.0", "bestValue": True},
                {"metric": "test_errors", "value": "0", "bestValue": True},
                {
                    "metric": "duplicated_lines_density",
                    "value": "0.0",
                    "bestValue": True,
                },
                {"metric": "test_failures", "value": "0", "bestValue": True},
            ],
        },
        {
            "id": "AX9Fg6R0WRlRH47OejaP",
            "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:measuresoftgram/cli.py",
            "name": "cli.py",
            "qualifier": "FIL",
            "path": "measuresoftgram/cli.py",
            "language": "py",
            "measures": [
                {"metric": "complexity", "value": "2"},
                {"metric": "functions", "value": "2"},
                {"metric": "ncloc", "value": "4"},
                {"metric": "coverage", "value": "100.0", "bestValue": True},
                {"metric": "security_rating", "value": "1.0", "bestValue": True},
                {
                    "metric": "comment_lines_density",
                    "value": "20.0",
                    "bestValue": False,
                },
                {"metric": "files", "value": "1"},
                {"metric": "test_errors", "value": "0", "bestValue": True},
                {
                    "metric": "duplicated_lines_density",
                    "value": "0.0",
                    "bestValue": True,
                },
                {"metric": "test_failures", "value": "0", "bestValue": True},
            ],
        },
        {
            "id": "AX9GDsKlZuVL7NjXSAZ3",
            "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:tests/hello_world_test.py",
            "name": "hello_world_test.py",
            "qualifier": "UTS",
            "path": "tests/hello_world_test.py",
            "language": "py",
            "measures": [
                {"metric": "security_rating", "value": "1.0", "bestValue": True},
                {"metric": "test_execution_time", "value": "2"},
                {"metric": "tests", "value": "2"},
                {"metric": "test_errors", "value": "0", "bestValue": True},
                {
                    "metric": "duplicated_lines_density",
                    "value": "0.0",
                    "bestValue": True,
                },
                {"metric": "test_failures", "value": "0", "bestValue": True},
            ],
        },
        {
            "id": "AX9Fg6R0WRlRH47OejaQ",
            "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:measuresoftgram",
            "name": "measuresoftgram",
            "qualifier": "DIR",
            "path": "measuresoftgram",
            "measures": [
                {
                    "metric": "duplicated_lines_density",
                    "value": "0.0",
                    "bestValue": True,
                },
                {"metric": "functions", "value": "2"},
                {"metric": "security_rating", "value": "1.0", "bestValue": True},
                {"metric": "files", "value": "1"},
                {"metric": "complexity", "value": "2"},
                {"metric": "ncloc", "value": "4"},
                {"metric": "coverage", "value": "100.0", "bestValue": True},
                {
                    "metric": "comment_lines_density",
                    "value": "20.0",
                    "bestValue": False,
                },
            ],
        },
        {
            "id": "AX9GDsKlZuVL7NjXSAZ5",
            "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:tests",
            "name": "tests",
            "qualifier": "DIR",
            "path": "tests",
            "measures": [
                {"metric": "test_execution_time", "value": "2"},
                {"metric": "test_failures", "value": "0", "bestValue": True},
                {"metric": "test_errors", "value": "0", "bestValue": True},
                {"metric": "security_rating", "value": "1.0", "bestValue": True},
                {"metric": "tests", "value": "2"},
            ],
        },
    ]

    components = jsonReader.file_reader("tests/unit/data/sonar.json")

    assert components == EXPECTED_SONAR_JSON_COMPONENTS


class TestOpenJsonFile:
    """
    Tests open_json_file function
    """

    def test_file_not_found(self):
        """
        Test when the file does not exists
        """

        with pytest.raises(exceptions.FileNotFound) as error:
            jsonReader.open_json_file("tests/utils/sona.json")

        assert str(error.value) == "The file was not found"

    def test_file_invalid_json(self):
        """
        Test when the file is an invalid JSON
        """

        with pytest.raises(exceptions.InvalidMetricsJsonFile) as error:
            jsonReader.open_json_file("tests/unit/data/invalid_json.json")

        assert "Failed to decode the JSON file." in str(error.value)


class TestValidateMetricsPost:
    """
    Tests validate_metrics_post function
    """

    @pytest.mark.parametrize("status_code", [200, 201, 204])
    def test_post_success(self, status_code):
        """
        Test when a success status code is returned
        """

        # with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        message = jsonReader.validate_metrics_post(status_code)

        assert "OK: Data sent successfully" == message

    @pytest.mark.parametrize("status_code", [400, 500, 404, 422])
    def test_post_failure(self, status_code):
        """
        Test when an error is returned
        """

        # with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        message = jsonReader.validate_metrics_post(status_code)

        assert (
            f'FAIL: The host service server returned a {status_code} error. Trying again'
            == message
        )


class TestCheckMetricsValues:
    """
    Tests check_metrics_values function
    """

    INVALID_CHECK_METRICS_VALUES_PARAMS = [
        (
            {
                "components": [
                    {
                        "key": "MeasureSoftGram-CLI:tests/__init__.py",
                        "measures": [
                            {
                                "metric": "security_rating",
                                "value": "1.0",
                            },
                            {
                                "metric": "test_errors",
                                "value": None,
                            },
                        ],
                    }
                ]
            },
            'Invalid metric value in "MeasureSoftGram-CLI:tests/__init__.py" component for the "test_errors" metric',
        ),
        (
            {
                "components": [
                    {
                        "key": "MeasureSoftGram-CLI:tests/__init__.py",
                        "measures": [
                            {
                                "metric": "security_rating",
                                "value": "1.0",
                            },
                            {
                                "metric": "test_failures",
                                "value": "",
                            },
                        ],
                    }
                ]
            },
            'Invalid metric value in "MeasureSoftGram-CLI:tests/__init__.py" component for the "test_failures" metric',
        ),
        (
            {
                "components": [
                    {
                        "key": "MeasureSoftGram-CLI:tests/__init__.py",
                        "measures": [
                            {
                                "metric": "files",
                                "value": "NaN",
                            },
                            {
                                "metric": "test_failures",
                                "value": "0.0",
                            },
                        ],
                    }
                ]
            },
            'Invalid metric value in "MeasureSoftGram-CLI:tests/__init__.py" component for the "files" metric',
        ),
    ]

    @pytest.mark.parametrize(
        "json_data, error_msg", INVALID_CHECK_METRICS_VALUES_PARAMS
    )
    def test_invalid_metric_value(self, json_data, error_msg):
        """
        Test invalid metric values
        """

        with pytest.raises(exceptions.InvalidMetricException) as error:
            jsonReader.check_metrics_values(json_data)

        assert error_msg in str(error.value)

    @pytest.mark.parametrize(
        "json_data",
        [
            {},
            {"components": [{}]},
            {"components": [{"measures": [{"metric": "a"}]}]},
        ],
    )
    def test_key_error(self, json_data):
        """
        Test invalid JSON
        """

        with pytest.raises(exceptions.InvalidMetricsJsonFile) as error:
            jsonReader.check_metrics_values(json_data)

        assert (
            "Failed to validate Sonar JSON metrics. Please check if the file is a valid Sonar JSON"
            in str(error.value)
        )


class TestCheckSonarFormat:
    """
    Tests check_sonar_format function
    """

    INVALID_CHECK_SONAR_FORMAT_PARAMS = [
        (
            {},
            "Invalid Sonar JSON keys. Missing keys are: paging, baseComponent, components",
        ),
        (
            {"baseComponent": {}, "paging": {}},
            "Invalid Sonar JSON keys. Missing keys are: components",
        ),
        (
            {
                "paging": {},
                "baseComponent": {"qualifier": "qualifier"},
                "components": {},
            },
            "Invalid Sonar baseComponent keys. Missing keys are: id, key, name, measures",
        ),
        (
            {
                "paging": {},
                "baseComponent": {
                    "id": "",
                    "key": "",
                    "name": "",
                    "qualifier": "",
                    "measures": "",
                },
                "components": {},
            },
            "Invalid Sonar JSON components value. It must have at least one component",
        ),
    ]

    @pytest.mark.parametrize("json_data, error_msg", INVALID_CHECK_SONAR_FORMAT_PARAMS)
    def test_check_sonar_format_invalid_json(self, json_data, error_msg):
        """
        Test invalid Sonar JSON data
        """

        with pytest.raises(exceptions.InvalidMetricsJsonFile) as error:
            jsonReader.check_sonar_format(json_data)

        assert error_msg in str(error.value)



import os
import argparse
import sys
import signal
from pathlib import Path

from src.cli.commands import parse_import, parse_get_entity

from src.config.settings import AVAILABLE_ENTITIES, AVAILABLE_IMPORTS, SUPPORTED_FORMATS


def sigint_handler(*_):
    print("\n\nExiting MeasureSoftGram...")
    sys.exit(0)


def setup():
    parser = argparse.ArgumentParser(
        description="Command line interface for measuresoftgram"
    )

    argparse.ArgumentTypeError('invalid value!!!')

    subparsers = parser.add_subparsers(dest="command", help="sub-command help")

    parser_import = subparsers.add_parser(
        "import",
        help="Import a folder with metrics"
    )

    parser_import.add_argument(
        "output_origin",
        type=str,
        choices=(AVAILABLE_IMPORTS),
        help=(
            "Import a metrics files from some origin. Valid values are: "
            + ", ".join(AVAILABLE_IMPORTS)
        ),
    )

    parser_import.add_argument(
        "dir_path",
        type=lambda p: Path(p).absolute(),
        default=Path(__file__).absolute().parent / "data",
        help="Path to the directory",
    )

    parser_import.add_argument(
        "language_extension",
        type=str,
        help="The source code language extension",
    )

    parser_import.add_argument(
        "--host",
        type=str,
        nargs='?',
        default=os.getenv(
            "MSG_SERVICE_HOST",
            "https://measuresoftgram-service.herokuapp.com/"
        ),
        help="The host of the service",
    )

    parser_import.add_argument(
        "--organization_id",
        type=str,
        nargs='?',
        default=os.getenv("MSG_ORGANIZATION_ID", "1"),
        help="The ID of the organization that the repository belongs to",
    )

    parser_import.add_argument(
        "--repository_id",
        type=str,
        nargs='?',
        default=os.getenv("MSG_REPOSITORY_ID", "1"),
        help="The ID of the repository",
    )

    parser_get_entity = subparsers.add_parser(
        "get",
    )

    parser_get_entity.add_argument(
        "entity",
        type=str,
        choices=(AVAILABLE_ENTITIES),
        help=(
            "The entity to get. Valid values are: "
            + ", ".join(AVAILABLE_ENTITIES)
        ),
    )

    parser_get_entity.add_argument(
        "entity_id",
        type=int,
        nargs='?',
        help=(
            "The ID of the entity to get. If not provided, a list with the "
            "last record of all available entities will be returned."
        ),
    )

    parser_get_entity.add_argument(
        "--history",
        action="store_true",
        default=False,
        help="The history of the repository",
    )

    parser_get_entity.add_argument(
        "--host",
        type=str,
        nargs='?',
        default="https://measuresoftgram-service.herokuapp.com/",
        help="The host of the service",
    )

    parser_get_entity.add_argument(
        "--output_format",
        type=str,
        nargs='?',
        default="tabular",
        help=(
            "The format of the output. "
            "Valid values are: " + ", ".join(SUPPORTED_FORMATS)
        ),
    )

    parser_get_entity.add_argument(
        "--organization_id",
        type=str,
        nargs='?',
        default=os.getenv("MSG_ORGANIZATION_ID", "1"),
        help="The ID of the organization that the repository belongs to",
    )

    parser_get_entity.add_argument(
        "--repository_id",
        type=str,
        nargs='?',
        default=os.getenv("MSG_REPOSITORY_ID", "1"),
        help="The ID of the repository",
    )

    # parser_create = subparsers.add_parser(
    #     "create",
    #     help="Create a new model pre configuration from a JSON file",
    # )

    # subparsers.add_parser(
    #     "available",
    #     help="Shows all characteristics, sub-characteristics and measures available in measuresoftgram",
    # )

    # parser_create.add_argument(
    #     "path",
    #     type=lambda p: Path(p).absolute(),
    #     default=Path(__file__).absolute().parent / "data",
    #     help="Path to the JSON file",
    # )

    # parser_analysis = subparsers.add_parser("analysis", help="Get analysis result")
    # parser_analysis.add_argument(
    #     "id",
    # )
    # subparsers.add_parser("list", help="List all pre configurations")

    # parser_show = subparsers.add_parser(
    #     "show", help="Show all information of a pre configuration"
    # )

    # parser_show.add_argument(
    #     "pre_config_id",
    #     type=str,
    #     help="Pre config ID",
    # )

    # change_name = subparsers.add_parser(
    #     "change-name", help="Change pre configuration name"
    # )

    # change_name.add_argument(
    #     "pre_config_id",
    #     type=str,
    #     help="Pre config ID",
    # )

    # change_name.add_argument(
    #     "new_name",
    #     type=str,
    #     help="New pre configuration name",
    # )

    args = parser.parse_args()

    # if args is empty show help
    if not sys.argv[1:]:
        parser.print_help()
        return

    elif args.command == "import":
        parse_import(
            args.output_origin,
            args.dir_path,
            args.language_extension,
            args.host,
            args.organization_id,
            args.repository_id,
        )

    # elif args.command == "create":
    #     parse_create(args.path)

    # elif args.command == "analysis":
    #     parse_analysis(args.id)

    # elif args.command == "available":
    #     parse_available()

    # elif args.command == "list":
    #     parse_list()

    # elif args.command == "show":
    #     parse_show(args.pre_config_id)

    # elif args.command == "change-name":
    #     parse_change_name(args.pre_config_id, args.new_name)

    elif args.command == 'get':
        parse_get_entity(
            args.entity,
            args.entity_id,
            args.host,
            args.organization_id,
            args.repository_id,
            args.output_format,
            args.history
        )


def main():
    """Entry point for the application script"""

    signal.signal(signal.SIGINT, sigint_handler)

    setup()



import re
from io import StringIO
from src.cli.commands import parse_list


class DummyResponse:
    def __init__(self, status_code):
        self.status_code = status_code

    def json(self):
        return [
            {
                "_id": "62656d15f354349ee4abfc7b",
                "name": "pre-config-1",
                "created_at": "2022-04-24 15:30:29+00:00",
            },
            {
                "_id": "62656e79f354349ee4abfc7c",
                "name": "pre-config-2",
                "created_at": "2022-04-24 15:36:25+00:00",
            },
            {
                "_id": "62656e7ef354349ee4abfc7d",
                "name": "pre-config-3",
                "created_at": "2022-04-24 15:36:30+00:00",
            },
        ]


def test_pre_configs_list(mocker):
    mocker.patch("requests.get", return_value=DummyResponse(200))

    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        parse_list()

        output_lines = fake_out.getvalue().splitlines()

        assert len(output_lines) == 4

        header_regexp = re.compile(r"ID\s+Name\s+Created at\s+Metrics file")

        assert header_regexp.match(output_lines[0]) is not None

        line_regexp = re.compile(
            r"(?P<id>[0-9a-z]+)\s+(?P<name>[^\s]+)\s+"
            + r"(?P<created_at>\d{2,}/\d{2,}/\d{4,}\s\d{2,}:\d{2,}:\d{2,})\s+"
            + r"(?P<metrics_file>[^\s]+)"
        )

        expected_groups = [
            (
                "62656d15f354349ee4abfc7b",
                "pre-config-1",
                "04/24/2022 12:30:29",
                "-",
            ),
            (
                "62656e79f354349ee4abfc7c",
                "pre-config-2",
                "04/24/2022 12:36:25",
                "-",
            ),
            (
                "62656e7ef354349ee4abfc7d",
                "pre-config-3",
                "04/24/2022 12:36:30",
                "-",
            ),
        ]

        for i in range(1, 4):
            match_data = line_regexp.match(output_lines[i])

            assert (
                match_data is not None
            ), f"output_lines[{i}] does not match the expected line regexp"
            assert (
                match_data.groups() == expected_groups[i - 1]
            ), f"{match_data.groups()} != {expected_groups[i - 1]}"


def test_error_in_pre_config_list(mocker):
    mocker.patch("requests.get", return_value=DummyResponse(500))

    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        parse_list()

        assert (
            "Error: an error occurred while fetching your pre configurations"
            in fake_out.getvalue()
        )



import subprocess


def capture(command):
    proc = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    out, err = proc.communicate()
    return out, err, proc.returncode


def test_import_metrics_folder_not_found_exception_handling():
    out, _, returncode = capture(
        ["measuresoftgram", "import", "sonarqube", "sonar-output-fake", "py"]
    )

    assert returncode == 0
    assert "Error: The folder was not found" in out.decode("utf-8")



from src.cli.commands import parse_available
from io import StringIO


mocked_available = {
    "characteristics": {
        "reliability": {
            "name": "Reliability",
            "subcharacteristics": ["testing_status"],
        },
        "maintainability": {
            "name": "Maintainability",
            "subcharacteristics": ["modifiability"],
        },
    },
    "subcharacteristics": {
        "testing_status": {
            "name": "Testing Status",
            "measures": ["passed_tests", "test_builds", "test_coverage"],
            "characteristics": ["reliability"],
        },
        "modifiability": {
            "name": "Modifiability",
            "measures": [
                "non_complex_file_density",
                "commented_file_density",
                "duplication_absense",
            ],
            "characteristics": ["maintainability"],
        },
    },
    "measures": {
        "passed_tests": {
            "name": "Passed Tests",
            "subcharacteristics": ["testing_status"],
            "characteristics": ["reliability"],
            "metrics": ["test_success_density"],
        },
        "test_builds": {
            "name": "Test Builds",
            "subcharacteristics": ["testing_status"],
            "characteristics": ["reliability"],
            "metrics": ["tests", "test_execution_time"],
        },
        "test_coverage": {
            "name": "Test Coverage",
            "subcharacteristics": ["testing_status"],
            "characteristics": ["reliability"],
            "metrics": ["coverage"],
        },
        "non_complex_file_density": {
            "name": "Non complex file density",
            "subcharacteristics": ["modifiability"],
            "characteristics": ["maintainability"],
            "metrics": ["complexity", "functions"],
        },
        "commented_file_density": {
            "name": "Commented file density",
            "subcharacteristics": ["modifiability"],
            "characteristics": ["maintainability"],
            "metrics": ["comment_lines_density"],
        },
        "duplication_absense": {
            "name": "Duplication abscense",
            "subcharacteristics": ["modifiability"],
            "characteristics": ["maintainability"],
            "metrics": ["duplicated_lines_density"],
        },
    },
}


class DummyResponse:
    def __init__(self, status_code, mocked_data):
        self.status_code = status_code
        self.res = mocked_data

    def json(self):
        return self.res


# @mock.patch('requests.get', return_value=DummyResponse(200, mocked_available))
def test_parse_available(mocker):

    with mocker.patch("sys.stdout", new=StringIO()) as fake_out:
        mocker.patch(
            "requests.get",
            return_value=DummyResponse(200, mocked_available)
        )
        parse_available()

        expected_lines = [
            "These are all items available in the MeasureSoftGram database in the following order:",
            "Characteristics -> Subcharacteristics -> Measures -> Necessary Metrics",
            "You can use these items to create a pre configuration",
            "Maintainability:",
            "Modifiability:",
            "Commented file density:",
            "comment_lines_density",
            "Duplication abscense:",
            "duplicated_lines_density",
            "Non complex file density:",
            "complexity, functions",
            "Reliability:",
            "Testing Status:",
            "Passed Tests:",
            "test_success_density",
            "Test Builds:",
            "tests, test_execution_time",
            "Test Coverage:",
            "coverage",
        ]

        for line in expected_lines:
            assert line in fake_out.getvalue()



import requests
from src.cli.utils import pretty_date_str

BASE_URL = "http://localhost:5000/"


def parse_show(id):
    response = requests.get(
        BASE_URL + f"/pre-configs/{id}",
        headers={"Accept": "application/json"},
    )

    response_data = response.json()

    if 200 <= response.status_code <= 299:
        print(f"Name: {response_data['name']}")
        print(f"ID: {response_data['_id']}")
        print(f"Created at: {pretty_date_str(response_data['created_at'])}")

        print(
            "\nSelected levels. Ordered as characteristics -> subcharacteristics -> measures\n"
        )

        for key, char_data in response_data["characteristics"].items():
            print(f"{key} (weigth: {char_data['weight']})")

            for subchar in char_data["subcharacteristics"]:
                subchar_data = response_data["subcharacteristics"][subchar]

                print(f"\t{subchar} (weigth: {char_data['weights'][subchar]})")

                for measure in subchar_data["measures"]:
                    print(f"\t\t{measure} (weigth: {subchar_data['weights'][measure]})")

            print("\n")
    else:
        print("Error: ", response_data["error"])



import requests
from requests.adapters import HTTPAdapter, Retry


class ServiceClient:
    @staticmethod
    def configure_session(session):
        errors = [500, 501, 502, 503, 504, 505, 506, 507, 508, 509, 510, 511]
        retries = Retry(total=5, backoff_factor=5, status_forcelist=errors)
        # session.mount("http://", HTTPAdapter(max_retries=retries))
        session.mount("https://", HTTPAdapter(max_retries=retries))

    @staticmethod
    def make_get_request(url):
        with requests.Session() as session:
            ServiceClient.configure_session(session)
            return session.get(url)

    @staticmethod
    def make_post_request(url, payload):
        with requests.Session() as session:
            ServiceClient.configure_session(session)
            return session.post(url, json=payload)

    @staticmethod
    def get_entity(url):
        return ServiceClient.make_get_request(url)

    @staticmethod
    def import_file(url, payload):
        return ServiceClient.make_post_request(url, payload)



from setuptools import setup, find_packages

setup(
    name="measuresoftgram",
    long_description="CLI da ferramenta MeasureSoftGram",
    version="3.1.1",
    extras_require={"dev": ["pytest", "pytest-cov", "setuptools", "wheel"]},
    packages=find_packages(),
    install_requires=[
        "inquirer==2.8.0",
        "requests==2.28.1",
        "pytz",
        "tabulate==0.8.10",
        "termcolor==1.1.0"
    ],
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={"console_scripts": ["measuresoftgram=src.cli.cliRunner:main"]},
)



import os

SERVICE_URL = os.getenv("SERVICE_URL", "https://measuresoftgram-service.herokuapp.com/")

BASE_URL = "http://172.20.0.2:5000/"

AVAILABLE_ENTITIES = [
    "metrics",
    "measures",
    "subcharacteristics",
    "characteristics",
    # "sqc",
]

SUPPORTED_FORMATS = [
    "json",
    "tabular",
]

AVAILABLE_IMPORTS = [
    "sonarqube"
]



import requests

from src.cli.commands.parse_analysis.results import validade_analysis_response
from src.config.settings import BASE_URL


def parse_analysis(id):
    data = {"pre_config_id": id}
    response = requests.post(BASE_URL + "analysis", json=data)

    validade_analysis_response(response.status_code, response.json())
